// QuadTime.cpp : implementation of the QuadTime/QuadTimeDiff classes
//

#include <time.h>

#include <iomanip.h>

#include "QuadTime.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

enum time_sep_names {
	SEP_DAY_MON = 0,
	SEP_MON_YEAR,
	SEP_DAT_TIM,
	SEP_HOUR_MIN,
	SEP_MIN_SEC,
	SEP_SEC_MILLI,
	NumTimeSep };

enum time_sep_styles {
	SEP_STYLE_EUROPEAN,
	SEP_STYLE_US,
	NumTimeSepStyle };

static const char time_sep[NumTimeSepStyle][NumTimeSep] = {
	/* Euro */ { '.', '.', ' ', ':', ':', '.' },
	/* U.S. */ { '/', '/', ' ', ':', ':', '.' } };

/////////////////////////////////////////////////////////////////////////////
// QuadTimeDiff construction/destruction

// defined inline in QuadTime.h

/////////////////////////////////////////////////////////////////////////////
// QuadTimeDiff operations

ostream& operator<<(ostream& os, const QuadTimeDiff d)
{
	ULONG day,hour,minute,second,milli;
	long iosFlags;
	char fill;

	d.qTime.ToRelTime(day,hour,minute,second,milli);

	iosFlags = os.flags(ios::dec | ios::right);
	fill = os.fill('0');
	os << setw(0) << day    << time_sep[SEP_STYLE_EUROPEAN][SEP_DAT_TIM]
	   << setw(2) << hour   << time_sep[SEP_STYLE_EUROPEAN][SEP_HOUR_MIN]
	   << setw(2) << minute << time_sep[SEP_STYLE_EUROPEAN][SEP_MIN_SEC]
	   << setw(2) << second << time_sep[SEP_STYLE_EUROPEAN][SEP_SEC_MILLI]
	   << setw(3) << milli;
	os.fill(fill);
	os.flags(iosFlags);

	return os;
}

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// QuadTime construction/destruction

// defined inline in QuadTime.h

/////////////////////////////////////////////////////////////////////////////
// QuadTime operations

ostream& operator<<(ostream& os, const QuadTime t)
{
	ULONG day,month,year,hour,minute,second,milli;
	long iosFlags;
	char fill;

	t.qTime.ToAbsTime(day,month,year,hour,minute,second,milli);

	iosFlags = os.flags(ios::dec | ios::right);
	fill = os.fill('0');
	// !! day and month are to be switched for SEP_STYLE_US !!
	os << setw(2) << day    << time_sep[SEP_STYLE_EUROPEAN][SEP_DAY_MON]
	   << setw(2) << month  << time_sep[SEP_STYLE_EUROPEAN][SEP_MON_YEAR]
	   << setw(4) << year   << time_sep[SEP_STYLE_EUROPEAN][SEP_DAT_TIM]
	   << setw(2) << hour   << time_sep[SEP_STYLE_EUROPEAN][SEP_HOUR_MIN]
	   << setw(2) << minute << time_sep[SEP_STYLE_EUROPEAN][SEP_MIN_SEC]
	   << setw(2) << second << time_sep[SEP_STYLE_EUROPEAN][SEP_SEC_MILLI]
	   << setw(3) << milli;
	os.fill(fill);
	os.flags(iosFlags);

	return os;
}

static struct tm * getUTC( void )
{
#if defined(WIN32) || defined(__OS2__)
	_tzset();
#else
#	error Local time zone has to be set!
#endif

	time_t ltime;

	time( &ltime );
	return gmtime( &ltime );
}

void QuadTime::SetNow(void)
{
	struct tm *gmt = getUTC();
	*this = QuadTime(gmt->tm_mday, gmt->tm_mon+1, 1900+gmt->tm_year,
	                 gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
}

void QuadTime::SetToday(void)
{
	struct tm *gmt = getUTC();
	*this = QuadTime(gmt->tm_mday, gmt->tm_mon+1, 1900+gmt->tm_year);
}

/////////////////////////////////////////////////////////////////////////////
