/*
** Test for the Quad, QuadTime und QuadTimeDiff functions and operators
*/

#include <stdlib.h>
#include <time.h>
#include <assert.h>

#include <iostream.h>
#include <iomanip.h>

#include "QuadTime.h"

int main(void)
{
	Quad         q1, q2, q3;
	QuadTime     t1, t2, t3;
	QuadTimeDiff d1, d2, d3;
	ULONG day,month,year,hour,minute,second,milli;

	// Quad(ULONG), QuadTime(Quad), QuadTime::operator=()
	t1 = QuadTime(Quad(0));
	// QuadTime::SetNow()
	t2.SetNow();
	// QuadTime(date)
	t3 = QuadTime(17,11,1858);

	// QuadTime::operator==(), QuadTime::operator!=()
	if (!(t1 == t3) || t1 != t3)
		throw "Null dates are not equal!";

	// QuadTime::operator<<()
	cout << "Null time : " << t1           << endl
	     << "Now       : " << t2 << " UTC" << endl;

	// QuadTime::operator-()
	d1 = d2 = t2 - t1;

	// QuadTimeDiff::operator/=(ULONG)
	d2 /= 2;

	// QuadTimeDiff::operator<<()
	cout << "Difference: " << d1 << endl
	     << "     / 2  : " << d2 << endl;

	// QuadTimeDiff::operator==(), QuadTimeDiff::operator!=()
	// QuadTimeDiff::operator/(ULONG)
	if (!(d2 == d1 / 2) || d2 != d1 / 2)
		throw "Divided differences are not equal!";

	// QuadTimeDiff::operator*(ULONG)
	if (d1 != d2 * 2 || d1 != 2 * d2)
		throw "Multiplied differences are not equal!";

	// QuadTime::Split()
	t2.Split(day,month,year,hour,minute,second,milli);
	cout << "Control: " << t2 << " ="
	     << " T:" << day
	     << " M:" << month
	     << " J:" << year
	     << " h:" << hour
	     << " m:" << minute
	     << " s:" << second
	     << " t:" << milli << endl;

	// QuadTimeDiff::Split()
	d2.Split(day,hour,minute,second,milli);
	cout << "         " << d2 << " ="
	     << " T:" << day
	     << " h:" << hour
	     << " m:" << minute
	     << " s:" << second
	     << " t:" << milli << endl;

	// 2. QuadTimeDiff::operator/(), QuadTimeDiff::operator%()
	d1 = QuadTimeDiff(2);
	d2 = QuadTimeDiff(0,1,0,1);
	q1 = d1 / d2;
	d3 = d1 % d2;

	// Quad::operator<<()
	cout << "Division : "
	     << d1 << " / " << d2 << " = "
	     << q1 << " Rest " << d3 << endl;

	//srand( (unsigned)time( NULL ) );

	assert( RAND_MAX == 0x7FFF );

	q1 = Quad(rand() << 16 | rand());
	q2 = Quad(rand() << 16 | rand());

	if ((q1 + q2) - q2 != q1)
		throw "Quad Addition/Subtraction is flawed!";

	if ((q1 * q2) / q2 != q1)
		throw "Quad Multiplication/Division is flawed!";

	return 0;
}
